@@ -1,10 +1,10 @@
 /* du -- summarize disk usage
-   Copyright (C) 1988-1991, 1995-2006 Free Software Foundation, Inc.
+   Copyright (C) 1988-1991, 1995-2010 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Differences from the Unix du:
    * Doesn't simply ignore the names of regular files given as arguments
@@ -25,23 +24,22 @@
    Rewritten to use nftw, then to use fts by Jim Meyering.  */
 
 #include <config.h>
-#include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include <assert.h>
 #include "system.h"
 #include "argmatch.h"
+#include "argv-iter.h"
 #include "error.h"
 #include "exclude.h"
 #include "fprintftime.h"
 #include "hash.h"
 #include "human.h"
-#include "inttostr.h"
 #include "quote.h"
 #include "quotearg.h"
-#include "readtokens0.h"
 #include "same.h"
 #include "stat-time.h"
+#include "stdio--.h"
 #include "xfts.h"
 #include "xstrtol.h"
 
@@ -51,7 +49,10 @@ extern bool fts_debug;
 #define PROGRAM_NAME "du"
 
 #define AUTHORS \
-  "Torbjorn Granlund", "David MacKenzie, Paul Eggert", "Jim Meyering"
+  proper_name_utf8 ("Torbjorn Granlund", "Torbj\303\266rn Granlund"), \
+  proper_name ("David MacKenzie"), \
+  proper_name ("Paul Eggert"), \
+  proper_name ("Jim Meyering")
 
 #if DU_DEBUG
 # define FTS_CROSS_CHECK(Fts) fts_cross_check (Fts)
@@ -66,7 +67,6 @@ extern bool fts_debug;
 
 /* Hash structure for inode and device numbers.  The separate entry
    structure makes it easier to rehash "in place".  */
-
 struct entry
 {
   ino_t st_ino;
@@ -124,9 +124,6 @@ struct dulevel
   struct duinfo subdir;
 };
 
-/* Name under which this program was invoked.  */
-char *program_name;
-
 /* If true, display counts for all files, not just directories.  */
 static bool opt_all = false;
 
@@ -195,15 +192,7 @@ enum
   EXCLUDE_OPTION,
   FILES0_FROM_OPTION,
   HUMAN_SI_OPTION,
-
-  /* FIXME: --kilobytes is deprecated (but not -k); remove in late 2006 */
-  KILOBYTES_LONG_OPTION,
-
   MAX_DEPTH_OPTION,
-
-  /* FIXME: --megabytes is deprecated (but not -m); remove in late 2006 */
-  MEGABYTES_LONG_OPTION,
-
   TIME_OPTION,
   TIME_STYLE_OPTION
 };
@@ -222,10 +211,8 @@ static struct option const long_options[] =
   {"files0-from", required_argument, NULL, FILES0_FROM_OPTION},
   {"human-readable", no_argument, NULL, 'h'},
   {"si", no_argument, NULL, HUMAN_SI_OPTION},
-  {"kilobytes", no_argument, NULL, KILOBYTES_LONG_OPTION},
   {"max-depth", required_argument, NULL, MAX_DEPTH_OPTION},
   {"null", no_argument, NULL, '0'},
-  {"megabytes", no_argument, NULL, MEGABYTES_LONG_OPTION},
   {"no-dereference", no_argument, NULL, 'P'},
   {"one-file-system", no_argument, NULL, 'x'},
   {"separate-dirs", no_argument, NULL, 'S'},
@@ -273,7 +260,7 @@ usage (int status)
 {
   if (status != EXIT_SUCCESS)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
+             program_name);
   else
     {
       printf (_("\
@@ -295,16 +282,17 @@ Mandatory arguments to long options are mandatory for short options too.\n\
                           fragmentation, indirect blocks, and the like\n\
 "), stdout);
       fputs (_("\
-  -B, --block-size=SIZE use SIZE-byte blocks\n\
+  -B, --block-size=SIZE  use SIZE-byte blocks\n\
   -b, --bytes           equivalent to `--apparent-size --block-size=1'\n\
   -c, --total           produce a grand total\n\
-  -D, --dereference-args  dereference FILEs that are symbolic links\n\
+  -D, --dereference-args  dereference only symlinks that are listed on the\n\
+                          command line\n\
 "), stdout);
       fputs (_("\
       --files0-from=F   summarize disk usage of the NUL-terminated file\n\
-                          names specified in file F\n\
-  -H                    like --si, but also evokes a warning; will soon\n\
-                          change to be equivalent to --dereference-args (-D)\n\
+                          names specified in file F;\n\
+                          If F is - then read names from standard input\n\
+  -H                    equivalent to --dereference-args (-D)\n\
   -h, --human-readable  print sizes in human readable format (e.g., 1K 234M 2G)\n\
       --si              like -h, but use powers of 1000 not 1024\n\
 "), stdout);
@@ -321,9 +309,9 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   -s, --summarize       display only a total for each argument\n\
 "), stdout);
       fputs (_("\
-  -x, --one-file-system  skip directories on different file systems\n\
-  -X FILE, --exclude-from=FILE  Exclude files that match any pattern in FILE.\n\
-      --exclude=PATTERN Exclude files that match PATTERN.\n\
+  -x, --one-file-system    skip directories on different file systems\n\
+  -X, --exclude-from=FILE  exclude files that match any pattern in FILE\n\
+      --exclude=PATTERN    exclude files that match PATTERN\n\
       --max-depth=N     print the total for a directory (or file, with --all)\n\
                           only if it is N or fewer levels below the command\n\
                           line argument;  --max-depth=0 is the same as\n\
@@ -334,17 +322,15 @@ Mandatory arguments to long options are mandatory for short options too.\n\
                           directory, or any of its subdirectories\n\
       --time=WORD       show time as WORD instead of modification time:\n\
                           atime, access, use, ctime or status\n\
-      --time-style=STYLE show times using style STYLE:\n\
+      --time-style=STYLE  show times using style STYLE:\n\
                           full-iso, long-iso, iso, +FORMAT\n\
                           FORMAT is interpreted like `date'\n\
 "), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
-      fputs (_("\n\
-SIZE may be (or may be an integer optionally followed by) one of following:\n\
-kB 1000, K 1024, MB 1000*1000, M 1024*1024, and so on for G, T, P, E, Z, Y.\n\
-"), stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_blocksize_note ("DU");
+      emit_size_note ();
+      emit_ancillary_info ();
     }
   exit (status);
 }
@@ -406,7 +392,7 @@ static void
 hash_init (void)
 {
   htab = hash_initialize (INITIAL_TABLE_SIZE, NULL,
-			  entry_hash, entry_compare, free);
+                          entry_hash, entry_compare, free);
   if (htab == NULL)
     xalloc_die ();
 }
@@ -422,10 +408,7 @@ show_date (const char *format, struct timespec when)
   if (! tm)
     {
       char buf[INT_BUFSIZE_BOUND (intmax_t)];
-      error (0, 0, _("time %s is out of range"),
-	     (TYPE_SIGNED (time_t)
-	      ? imaxtostr (when.tv_sec, buf)
-	      : umaxtostr (when.tv_sec, buf)));
+      error (0, 0, _("time %s is out of range"), timetostr (when.tv_sec, buf));
       fputs (buf, stdout);
       return;
     }
@@ -440,7 +423,7 @@ print_only_size (uintmax_t n_bytes)
 {
   char buf[LONGEST_HUMAN_READABLE + 1];
   fputs (human_readable (n_bytes, buf, human_output_opts,
-			 1, output_block_size), stdout);
+                         1, output_block_size), stdout);
 }
 
 /* Print size (and optionally time) indicated by *PDUI, followed by STRING.  */
@@ -488,7 +471,7 @@ process_file (FTS *fts, FTSENT *ent)
   bool skip;
 
   /* If necessary, set FTS_SKIP before returning.  */
-  skip = excluded_file_name (exclude, ent->fts_path);
+  skip = excluded_file_name (exclude, file);
   if (skip)
     fts_set (fts, ent, FTS_SKIP);
 
@@ -505,11 +488,20 @@ process_file (FTS *fts, FTSENT *ent)
 
     case FTS_DNR:
       /* Don't return just yet, since although the directory is not readable,
-	 we were able to stat it, so we do have a size.  */
+         we were able to stat it, so we do have a size.  */
       error (0, ent->fts_errno, _("cannot read directory %s"), quote (file));
       ok = false;
       break;
 
+    case FTS_DC:		/* directory that causes cycles */
+      if (cycle_warning_required (fts, ent))
+        {
+          emit_cycle_warning (file);
+          return false;
+        }
+      ok = true;
+      break;
+
     default:
       ok = true;
       break;
@@ -525,25 +517,25 @@ process_file (FTS *fts, FTSENT *ent)
      via a hard link, then don't let it contribute to the sums.  */
   if (skip
       || (!opt_count_all
-	  && ! S_ISDIR (sb->st_mode)
-	  && 1 < sb->st_nlink
-	  && ! hash_ins (sb->st_ino, sb->st_dev)))
+          && ! S_ISDIR (sb->st_mode)
+          && 1 < sb->st_nlink
+          && ! hash_ins (sb->st_ino, sb->st_dev)))
     {
       /* Note that we must not simply return here.
-	 We still have to update prev_level and maybe propagate
-	 some sums up the hierarchy.  */
+         We still have to update prev_level and maybe propagate
+         some sums up the hierarchy.  */
       duinfo_init (&dui);
       print = false;
     }
   else
     {
       duinfo_set (&dui,
-		  (apparent_size
-		   ? sb->st_size
-		   : (uintmax_t) ST_NBLOCKS (*sb) * ST_NBLOCKSIZE),
-		  (time_type == time_mtime ? get_stat_mtime (sb)
-		   : time_type == time_atime ? get_stat_atime (sb)
-		   : get_stat_ctime (sb)));
+                  (apparent_size
+                   ? sb->st_size
+                   : (uintmax_t) ST_NBLOCKS (*sb) * ST_NBLOCKSIZE),
+                  (time_type == time_mtime ? get_stat_mtime (sb)
+                   : time_type == time_atime ? get_stat_atime (sb)
+                   : get_stat_ctime (sb)));
     }
 
   level = ent->fts_level;
@@ -557,44 +549,44 @@ process_file (FTS *fts, FTSENT *ent)
   else
     {
       if (level == prev_level)
-	{
-	  /* This is usually the most common case.  Do nothing.  */
-	}
+        {
+          /* This is usually the most common case.  Do nothing.  */
+        }
       else if (level > prev_level)
-	{
-	  /* Descending the hierarchy.
-	     Clear the accumulators for *all* levels between prev_level
-	     and the current one.  The depth may change dramatically,
-	     e.g., from 1 to 10.  */
-	  size_t i;
+        {
+          /* Descending the hierarchy.
+             Clear the accumulators for *all* levels between prev_level
+             and the current one.  The depth may change dramatically,
+             e.g., from 1 to 10.  */
+          size_t i;
 
-	  if (n_alloc <= level)
-	    {
-	      dulvl = xnrealloc (dulvl, level, 2 * sizeof *dulvl);
-	      n_alloc = level * 2;
-	    }
+          if (n_alloc <= level)
+            {
+              dulvl = xnrealloc (dulvl, level, 2 * sizeof *dulvl);
+              n_alloc = level * 2;
+            }
 
-	  for (i = prev_level + 1; i <= level; i++)
-	    {
-	      duinfo_init (&dulvl[i].ent);
-	      duinfo_init (&dulvl[i].subdir);
-	    }
-	}
+          for (i = prev_level + 1; i <= level; i++)
+            {
+              duinfo_init (&dulvl[i].ent);
+              duinfo_init (&dulvl[i].subdir);
+            }
+        }
       else /* level < prev_level */
-	{
-	  /* Ascending the hierarchy.
-	     Process a directory only after all entries in that
-	     directory have been processed.  When the depth decreases,
-	     propagate sums from the children (prev_level) to the parent.
-	     Here, the current level is always one smaller than the
-	     previous one.  */
-	  assert (level == prev_level - 1);
-	  duinfo_add (&dui_to_print, &dulvl[prev_level].ent);
-	  if (!opt_separate_dirs)
-	    duinfo_add (&dui_to_print, &dulvl[prev_level].subdir);
-	  duinfo_add (&dulvl[level].subdir, &dulvl[prev_level].ent);
-	  duinfo_add (&dulvl[level].subdir, &dulvl[prev_level].subdir);
-	}
+        {
+          /* Ascending the hierarchy.
+             Process a directory only after all entries in that
+             directory have been processed.  When the depth decreases,
+             propagate sums from the children (prev_level) to the parent.
+             Here, the current level is always one smaller than the
+             previous one.  */
+          assert (level == prev_level - 1);
+          duinfo_add (&dui_to_print, &dulvl[prev_level].ent);
+          if (!opt_separate_dirs)
+            duinfo_add (&dui_to_print, &dulvl[prev_level].subdir);
+          duinfo_add (&dulvl[level].subdir, &dulvl[prev_level].ent);
+          duinfo_add (&dulvl[level].subdir, &dulvl[prev_level].subdir);
+        }
     }
 
   prev_level = level;
@@ -605,14 +597,9 @@ process_file (FTS *fts, FTSENT *ent)
     duinfo_add (&dulvl[level].ent, &dui);
 
   /* Even if this directory is unreadable or we can't chdir into it,
-     do let its size contribute to the total, ... */
+     do let its size contribute to the total. */
   duinfo_add (&tot_dui, &dui);
 
-  /* ... but don't print out a total for it, since without the size(s)
-     of any potential entries, it could be very misleading.  */
-  if (ent->fts_info == FTS_DNR)
-    return ok;
-
   /* If we're not counting an entry, e.g., because it's a hard link
      to a file we've already counted (and --count-links), then don't
      print a line for it.  */
@@ -641,50 +628,45 @@ du_files (char **files, int bit_flags)
       FTS *fts = xfts_open (files, bit_flags, NULL);
 
       while (1)
-	{
-	  FTSENT *ent;
+        {
+          FTSENT *ent;
 
-	  ent = fts_read (fts);
-	  if (ent == NULL)
-	    {
-	      if (errno != 0)
-		{
-		  /* FIXME: try to give a better message  */
-		  error (0, errno, _("fts_read failed"));
-		  ok = false;
-		}
-	      break;
-	    }
-	  FTS_CROSS_CHECK (fts);
+          ent = fts_read (fts);
+          if (ent == NULL)
+            {
+              if (errno != 0)
+                {
+                  /* FIXME: try to give a better message  */
+                  error (0, errno, _("fts_read failed"));
+                  ok = false;
+                }
+              break;
+            }
+          FTS_CROSS_CHECK (fts);
 
-	  ok &= process_file (fts, ent);
-	}
+          ok &= process_file (fts, ent);
+        }
 
-      /* Ignore failure, since the only way it can do so is in failing to
-	 return to the original directory, and since we're about to exit,
-	 that doesn't matter.  */
-      fts_close (fts);
+      if (fts_close (fts) != 0)
+        {
+          error (0, errno, _("fts_close failed"));
+          ok = false;
+        }
     }
 
-  if (print_grand_total)
-    print_size (&tot_dui, _("total"));
-
   return ok;
 }
 
 int
 main (int argc, char **argv)
 {
-  int c;
   char *cwd_only[2];
   bool max_depth_specified = false;
-  char **files;
   bool ok = true;
   char *files_from = NULL;
-  struct Tokens tok;
 
   /* Bit flags that control how fts works.  */
-  int bit_flags = FTS_TIGHT_CYCLE_CHECK;
+  int bit_flags = FTS_TIGHT_CYCLE_CHECK | FTS_DEFER_STAT;
 
   /* Select one of the three FTS_ options that control if/when
      to follow a symlink.  */
@@ -693,11 +675,11 @@ main (int argc, char **argv)
   /* If true, display only a total for each argument. */
   bool opt_summarize_only = false;
 
-  cwd_only[0] = ".";
+  cwd_only[0] = bad_cast (".");
   cwd_only[1] = NULL;
 
   initialize_main (&argc, &argv);
-  program_name = argv[0];
+  set_program_name (argv[0]);
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
@@ -706,167 +688,165 @@ main (int argc, char **argv)
 
   exclude = new_exclude ();
 
-  human_output_opts = human_options (getenv ("DU_BLOCK_SIZE"), false,
-				     &output_block_size);
+  human_options (getenv ("DU_BLOCK_SIZE"),
+                 &human_output_opts, &output_block_size);
 
-  while ((c = getopt_long (argc, argv, DEBUG_OPT "0abchHklmsxB:DLPSX:",
-			   long_options, NULL)) != -1)
+  for (;;)
     {
+      int oi = -1;
+      int c = getopt_long (argc, argv, DEBUG_OPT "0abchHklmsxB:DLPSX:",
+                           long_options, &oi);
+      if (c == -1)
+        break;
+
       switch (c)
-	{
+        {
 #if DU_DEBUG
-	case 'd':
-	  fts_debug = true;
-	  break;
+        case 'd':
+          fts_debug = true;
+          break;
 #endif
 
-	case '0':
-	  opt_nul_terminate_output = true;
-	  break;
+        case '0':
+          opt_nul_terminate_output = true;
+          break;
 
-	case 'a':
-	  opt_all = true;
-	  break;
+        case 'a':
+          opt_all = true;
+          break;
 
-	case APPARENT_SIZE_OPTION:
-	  apparent_size = true;
-	  break;
+        case APPARENT_SIZE_OPTION:
+          apparent_size = true;
+          break;
 
-	case 'b':
-	  apparent_size = true;
-	  human_output_opts = 0;
-	  output_block_size = 1;
-	  break;
+        case 'b':
+          apparent_size = true;
+          human_output_opts = 0;
+          output_block_size = 1;
+          break;
 
-	case 'c':
-	  print_grand_total = true;
-	  break;
+        case 'c':
+          print_grand_total = true;
+          break;
 
-	case 'h':
-	  human_output_opts = human_autoscale | human_SI | human_base_1024;
-	  output_block_size = 1;
-	  break;
+        case 'h':
+          human_output_opts = human_autoscale | human_SI | human_base_1024;
+          output_block_size = 1;
+          break;
 
-	case 'H':  /* FIXME: remove warning and move this "case 'H'" to
-		      precede --dereference-args in late 2006.  */
-	  error (0, 0, _("WARNING: use --si, not -H; the meaning of the -H\
- option will soon\nchange to be the same as that of --dereference-args (-D)"));
-	  /* fall through */
-	case HUMAN_SI_OPTION:
-	  human_output_opts = human_autoscale | human_SI;
-	  output_block_size = 1;
-	  break;
+        case HUMAN_SI_OPTION:
+          human_output_opts = human_autoscale | human_SI;
+          output_block_size = 1;
+          break;
 
-	case KILOBYTES_LONG_OPTION:
-	  error (0, 0,
-		 _("the --kilobytes option is deprecated; use -k instead"));
-	  /* fall through */
-	case 'k':
-	  human_output_opts = 0;
-	  output_block_size = 1024;
-	  break;
+        case 'k':
+          human_output_opts = 0;
+          output_block_size = 1024;
+          break;
 
-	case MAX_DEPTH_OPTION:		/* --max-depth=N */
-	  {
-	    unsigned long int tmp_ulong;
-	    if (xstrtoul (optarg, NULL, 0, &tmp_ulong, NULL) == LONGINT_OK
-		&& tmp_ulong <= SIZE_MAX)
-	      {
-		max_depth_specified = true;
-		max_depth = tmp_ulong;
-	      }
-	    else
-	      {
-		error (0, 0, _("invalid maximum depth %s"),
-		       quote (optarg));
-		ok = false;
-	      }
-	  }
-	  break;
+        case MAX_DEPTH_OPTION:		/* --max-depth=N */
+          {
+            unsigned long int tmp_ulong;
+            if (xstrtoul (optarg, NULL, 0, &tmp_ulong, NULL) == LONGINT_OK
+                && tmp_ulong <= SIZE_MAX)
+              {
+                max_depth_specified = true;
+                max_depth = tmp_ulong;
+              }
+            else
+              {
+                error (0, 0, _("invalid maximum depth %s"),
+                       quote (optarg));
+                ok = false;
+              }
+          }
+          break;
 
-	case MEGABYTES_LONG_OPTION:
-	  error (0, 0,
-		 _("the --megabytes option is deprecated; use -m instead"));
-	  /* fall through */
-	case 'm':
-	  human_output_opts = 0;
-	  output_block_size = 1024 * 1024;
-	  break;
+        case 'm':
+          human_output_opts = 0;
+          output_block_size = 1024 * 1024;
+          break;
 
-	case 'l':
-	  opt_count_all = true;
-	  break;
+        case 'l':
+          opt_count_all = true;
+          break;
 
-	case 's':
-	  opt_summarize_only = true;
-	  break;
+        case 's':
+          opt_summarize_only = true;
+          break;
 
-	case 'x':
-	  bit_flags |= FTS_XDEV;
-	  break;
+        case 'x':
+          bit_flags |= FTS_XDEV;
+          break;
 
-	case 'B':
-	  human_output_opts = human_options (optarg, true, &output_block_size);
-	  break;
+        case 'B':
+          {
+            enum strtol_error e = human_options (optarg, &human_output_opts,
+                                                 &output_block_size);
+            if (e != LONGINT_OK)
+              xstrtol_fatal (e, oi, c, long_options, optarg);
+          }
+          break;
 
-	case 'D': /* This will eventually be 'H' (-H), too.  */
-	  symlink_deref_bits = FTS_COMFOLLOW | FTS_PHYSICAL;
-	  break;
+        case 'H':  /* NOTE: before 2008-12, -H was equivalent to --si.  */
+        case 'D':
+          symlink_deref_bits = FTS_COMFOLLOW | FTS_PHYSICAL;
+          break;
 
-	case 'L': /* --dereference */
-	  symlink_deref_bits = FTS_LOGICAL;
-	  break;
+        case 'L': /* --dereference */
+          symlink_deref_bits = FTS_LOGICAL;
+          break;
 
-	case 'P': /* --no-dereference */
-	  symlink_deref_bits = FTS_PHYSICAL;
-	  break;
+        case 'P': /* --no-dereference */
+          symlink_deref_bits = FTS_PHYSICAL;
+          break;
 
-	case 'S':
-	  opt_separate_dirs = true;
-	  break;
+        case 'S':
+          opt_separate_dirs = true;
+          break;
 
-	case 'X':
-	  if (add_exclude_file (add_exclude, exclude, optarg,
-				EXCLUDE_WILDCARDS, '\n'))
-	    {
-	      error (0, errno, "%s", quotearg_colon (optarg));
-	      ok = false;
-	    }
-	  break;
+        case 'X':
+          if (add_exclude_file (add_exclude, exclude, optarg,
+                                EXCLUDE_WILDCARDS, '\n'))
+            {
+              error (0, errno, "%s", quotearg_colon (optarg));
+              ok = false;
+            }
+          break;
 
-	case FILES0_FROM_OPTION:
-	  files_from = optarg;
-	  break;
+        case FILES0_FROM_OPTION:
+          files_from = optarg;
+          break;
 
-	case EXCLUDE_OPTION:
-	  add_exclude (exclude, optarg, EXCLUDE_WILDCARDS);
-	  break;
+        case EXCLUDE_OPTION:
+          add_exclude (exclude, optarg, EXCLUDE_WILDCARDS);
+          break;
 
-	case TIME_OPTION:
-	  opt_time = true;
-	  time_type =
-	    (optarg
-	     ? XARGMATCH ("--time", optarg, time_args, time_types)
-	     : time_mtime);
-	  break;
+        case TIME_OPTION:
+          opt_time = true;
+          time_type =
+            (optarg
+             ? XARGMATCH ("--time", optarg, time_args, time_types)
+             : time_mtime);
+          break;
 
-	case TIME_STYLE_OPTION:
-	  time_style = optarg;
-	  break;
+        case TIME_STYLE_OPTION:
+          time_style = optarg;
+          break;
 
-	case_GETOPT_HELP_CHAR;
+        case_GETOPT_HELP_CHAR;
 
-	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
+        case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
 
-	default:
-	  ok = false;
-	}
+        default:
+          ok = false;
+        }
     }
 
   if (!ok)
     usage (EXIT_FAILURE);
 
-  if (opt_all & opt_summarize_only)
+  if (opt_all && opt_summarize_only)
     {
       error (0, 0, _("cannot both summarize and show all entries"));
       usage (EXIT_FAILURE);
@@ -875,7 +855,7 @@ main (int argc, char **argv)
   if (opt_summarize_only && max_depth_specified && max_depth == 0)
     {
       error (0, 0,
-	     _("warning: summarizing is the same as using --max-depth=0"));
+             _("warning: summarizing is the same as using --max-depth=0"));
     }
 
   if (opt_summarize_only && max_depth_specified && max_depth != 0)
@@ -892,128 +872,158 @@ main (int argc, char **argv)
   if (opt_time)
     {
       if (! time_style)
-	{
-	  time_style = getenv ("TIME_STYLE");
+        {
+          time_style = getenv ("TIME_STYLE");
 
-	  /* Ignore TIMESTYLE="locale", for compatibility with ls.  */
-	  if (! time_style || STREQ (time_style, "locale"))
-	    time_style = "long-iso";
-	  else if (*time_style == '+')
-	    {
-	      /* Ignore anything after a newline, for compatibility
-		 with ls.  */
-	      char *p = strchr (time_style, '\n');
-	      if (p)
-		*p = '\0';
-	    }
-	  else
-	    {
-	      /* Ignore "posix-" prefix, for compatibility with ls.  */
-	      static char const posix_prefix[] = "posix-";
-	      while (strncmp (time_style, posix_prefix, sizeof posix_prefix - 1)
-		     == 0)
-		time_style += sizeof posix_prefix - 1;
-	    }
-	}
+          /* Ignore TIMESTYLE="locale", for compatibility with ls.  */
+          if (! time_style || STREQ (time_style, "locale"))
+            time_style = "long-iso";
+          else if (*time_style == '+')
+            {
+              /* Ignore anything after a newline, for compatibility
+                 with ls.  */
+              char *p = strchr (time_style, '\n');
+              if (p)
+                *p = '\0';
+            }
+          else
+            {
+              /* Ignore "posix-" prefix, for compatibility with ls.  */
+              static char const posix_prefix[] = "posix-";
+              while (strncmp (time_style, posix_prefix, sizeof posix_prefix - 1)
+                     == 0)
+                time_style += sizeof posix_prefix - 1;
+            }
+        }
 
       if (*time_style == '+')
-	time_format = time_style + 1;
+        time_format = time_style + 1;
       else
         {
           switch (XARGMATCH ("time style", time_style,
                              time_style_args, time_style_types))
             {
-	    case full_iso_time_style:
-	      time_format = "%Y-%m-%d %H:%M:%S.%N %z";
-	      break;
+            case full_iso_time_style:
+              time_format = "%Y-%m-%d %H:%M:%S.%N %z";
+              break;
 
-	    case long_iso_time_style:
-	      time_format = "%Y-%m-%d %H:%M";
-	      break;
+            case long_iso_time_style:
+              time_format = "%Y-%m-%d %H:%M";
+              break;
 
-	    case iso_time_style:
-	      time_format = "%Y-%m-%d";
-	      break;
+            case iso_time_style:
+              time_format = "%Y-%m-%d";
+              break;
             }
         }
     }
 
+  struct argv_iterator *ai;
   if (files_from)
     {
       /* When using --files0-from=F, you may not specify any files
-	 on the command-line.  */
+         on the command-line.  */
       if (optind < argc)
-	{
-	  error (0, 0, _("extra operand %s"), quote (argv[optind]));
-	  fprintf (stderr, "%s\n",
-		   _("File operands cannot be combined with --files0-from."));
-	  usage (EXIT_FAILURE);
-	}
+        {
+          error (0, 0, _("extra operand %s"), quote (argv[optind]));
+          fprintf (stderr, "%s\n",
+                   _("file operands cannot be combined with --files0-from"));
+          usage (EXIT_FAILURE);
+        }
 
       if (! (STREQ (files_from, "-") || freopen (files_from, "r", stdin)))
-	error (EXIT_FAILURE, errno, _("cannot open %s for reading"),
-	       quote (files_from));
+        error (EXIT_FAILURE, errno, _("cannot open %s for reading"),
+               quote (files_from));
 
-      readtokens0_init (&tok);
-
-      if (! readtokens0 (stdin, &tok) || fclose (stdin) != 0)
-	error (EXIT_FAILURE, 0, _("cannot read file names from %s"),
-	       quote (files_from));
-
-      files = tok.tok;
+      ai = argv_iter_init_stream (stdin);
     }
   else
     {
-      files = (optind < argc ? argv + optind : cwd_only);
+      char **files = (optind < argc ? argv + optind : cwd_only);
+      ai = argv_iter_init_argv (files);
     }
 
+  if (!ai)
+    xalloc_die ();
+
   /* Initialize the hash structure for inode numbers.  */
   hash_init ();
 
-  /* Report and filter out any empty file names before invoking fts.
-     This works around a glitch in fts, which fails immediately
-     (without looking at the other file names) when given an empty
-     file name.  */
-  {
-    size_t i = 0;
-    size_t j;
-
-    for (j = 0; ; j++)
-      {
-	if (i != j)
-	  files[i] = files[j];
-
-	if ( ! files[i])
-	  break;
-
-	if (files[i][0])
-	  i++;
-	else
-	  {
-	    if (files_from)
-	      {
-		/* Using the standard `filename:line-number:' prefix here is
-		   not totally appropriate, since NUL is the separator, not NL,
-		   but it might be better than nothing.  */
-		unsigned long int file_number = j + 1;
-		error (0, 0, "%s:%lu: %s", quotearg_colon (files_from),
-		       file_number, _("invalid zero-length file name"));
-	      }
-	    else
-	      error (0, 0, "%s", _("invalid zero-length file name"));
-	  }
-      }
-
-    ok = (i == j);
-  }
-
   bit_flags |= symlink_deref_bits;
-  ok &= du_files (files, bit_flags);
+  static char *temp_argv[] = { NULL, NULL };
 
-  /* This isn't really necessary, but it does ensure we
-     exercise this function.  */
-  if (files_from)
-    readtokens0_free (&tok);
+  while (true)
+    {
+      bool skip_file = false;
+      enum argv_iter_err ai_err;
+      char *file_name = argv_iter (ai, &ai_err);
+      if (ai_err == AI_ERR_EOF)
+        break;
+      if (!file_name)
+        {
+          switch (ai_err)
+            {
+            case AI_ERR_READ:
+              error (0, errno, _("%s: read error"), quote (files_from));
+              continue;
+
+            case AI_ERR_MEM:
+              xalloc_die ();
+
+            default:
+              assert (!"unexpected error code from argv_iter");
+            }
+        }
+      if (files_from && STREQ (files_from, "-") && STREQ (file_name, "-"))
+        {
+          /* Give a better diagnostic in an unusual case:
+             printf - | du --files0-from=- */
+          error (0, 0, _("when reading file names from stdin, "
+                         "no file name of %s allowed"),
+                 quote (file_name));
+          skip_file = true;
+        }
+
+      /* Report and skip any empty file names before invoking fts.
+         This works around a glitch in fts, which fails immediately
+         (without looking at the other file names) when given an empty
+         file name.  */
+      if (!file_name[0])
+        {
+          /* Diagnose a zero-length file name.  When it's one
+             among many, knowing the record number may help.
+             FIXME: currently print the record number only with
+             --files0-from=FILE.  Maybe do it for argv, too?  */
+          if (files_from == NULL)
+            error (0, 0, "%s", _("invalid zero-length file name"));
+          else
+            {
+              /* Using the standard `filename:line-number:' prefix here is
+                 not totally appropriate, since NUL is the separator, not NL,
+                 but it might be better than nothing.  */
+              unsigned long int file_number = argv_iter_n_args (ai);
+              error (0, 0, "%s:%lu: %s", quotearg_colon (files_from),
+                     file_number, _("invalid zero-length file name"));
+            }
+          skip_file = true;
+        }
+
+      if (skip_file)
+        ok = false;
+      else
+        {
+          temp_argv[0] = file_name;
+          ok &= du_files (temp_argv, bit_flags);
+        }
+    }
+
+  argv_iter_free (ai);
+
+  if (files_from && (ferror (stdin) || fclose (stdin) != 0))
+    error (EXIT_FAILURE, 0, _("error reading %s"), quote (files_from));
+
+  if (print_grand_total)
+    print_size (&tot_dui, _("total"));
 
   hash_free (htab);
 
