@@ -1,10 +1,11 @@
 /* sum -- checksum and count the blocks in a file
-   Copyright (C) 86, 89, 91, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1989, 1991, 1995-2002, 2004-2005, 2008-2010 Free
+   Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Like BSD sum or SysV sum -r, except like SysV sum if -s option is given. */
 
@@ -28,17 +28,17 @@
 #include "error.h"
 #include "human.h"
 #include "safe-read.h"
+#include "xfreopen.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "sum"
 
-#define AUTHORS "Kayvan Aghaiepour", "David MacKenzie"
+#define AUTHORS \
+  proper_name ("Kayvan Aghaiepour"), \
+  proper_name ("David MacKenzie")
 
-/* The name this program was run with. */
-char *program_name;
-
-/* Nonzero if any of the files read were the standard input. */
-static int have_read_stdin;
+/* True if any of the files read were the standard input. */
+static bool have_read_stdin;
 
 static struct option const longopts[] =
 {
@@ -51,19 +51,19 @@ static struct option const longopts[] =
 void
 usage (int status)
 {
-  if (status != 0)
+  if (status != EXIT_SUCCESS)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
+             program_name);
   else
     {
       printf (_("\
 Usage: %s [OPTION]... [FILE]...\n\
 "),
-	      program_name);
+              program_name);
       fputs (_("\
 Print checksum and block counts for each FILE.\n\
 \n\
-  -r              defeat -s, use BSD sum algorithm, use 1K blocks\n\
+  -r              use BSD sum algorithm, use 1K blocks\n\
   -s, --sysv      use System V sum algorithm, use 512 bytes blocks\n\
 "), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
@@ -72,42 +72,43 @@ Print checksum and block counts for each FILE.\n\
 \n\
 With no FILE, or when FILE is -, read standard input.\n\
 "), stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_ancillary_info ();
     }
-  exit (status == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
+  exit (status);
 }
 
 /* Calculate and print the rotated checksum and the size in 1K blocks
    of file FILE, or of the standard input if FILE is "-".
    If PRINT_NAME is >1, print FILE next to the checksum and size.
-   The checksum varies depending on sizeof(int).
-   Return 0 if successful, -1 if an error occurs. */
+   The checksum varies depending on sizeof (int).
+   Return true if successful.  */
 
-static int
+static bool
 bsd_sum_file (const char *file, int print_name)
 {
-  register FILE *fp;
-  register int checksum = 0;	/* The checksum mod 2^16. */
-  register uintmax_t total_bytes = 0;	/* The number of bytes. */
-  register int ch;		/* Each character read. */
+  FILE *fp;
+  int checksum = 0;	/* The checksum mod 2^16. */
+  uintmax_t total_bytes = 0;	/* The number of bytes. */
+  int ch;		/* Each character read. */
   char hbuf[LONGEST_HUMAN_READABLE + 1];
+  bool is_stdin = STREQ (file, "-");
 
-  if (STREQ (file, "-"))
+  if (is_stdin)
     {
       fp = stdin;
-      have_read_stdin = 1;
+      have_read_stdin = true;
+      if (O_BINARY && ! isatty (STDIN_FILENO))
+        xfreopen (NULL, "rb", stdin);
     }
   else
     {
-      fp = fopen (file, "r");
+      fp = fopen (file, (O_BINARY ? "rb" : "r"));
       if (fp == NULL)
-	{
-	  error (0, errno, "%s", file);
-	  return -1;
-	}
+        {
+          error (0, errno, "%s", file);
+          return false;
+        }
     }
-  /* Need binary I/O, or else byte counts and checksums are incorrect.  */
-  SET_BINARY (fileno(fp));
 
   while ((ch = getc (fp)) != EOF)
     {
@@ -120,32 +121,32 @@ bsd_sum_file (const char *file, int print_name)
   if (ferror (fp))
     {
       error (0, errno, "%s", file);
-      if (!STREQ (file, "-"))
-	fclose (fp);
-      return -1;
+      if (!is_stdin)
+        fclose (fp);
+      return false;
     }
 
-  if (!STREQ (file, "-") && fclose (fp) == EOF)
+  if (!is_stdin && fclose (fp) != 0)
     {
       error (0, errno, "%s", file);
-      return -1;
+      return false;
     }
 
   printf ("%05d %5s", checksum,
-	  human_readable (total_bytes, hbuf, human_ceiling, 1, 1024));
+          human_readable (total_bytes, hbuf, human_ceiling, 1, 1024));
   if (print_name > 1)
     printf (" %s", file);
   putchar ('\n');
 
-  return 0;
+  return true;
 }
 
 /* Calculate and print the checksum and the size in 512-byte blocks
    of file FILE, or of the standard input if FILE is "-".
    If PRINT_NAME is >0, print FILE next to the checksum and size.
-   Return 0 if successful, -1 if an error occurs. */
+   Return true if successful.  */
 
-static int
+static bool
 sysv_sum_file (const char *file, int print_name)
 {
   int fd;
@@ -158,22 +159,24 @@ sysv_sum_file (const char *file, int print_name)
   /* The sum of all the input bytes, modulo (UINT_MAX + 1).  */
   unsigned int s = 0;
 
-  if (STREQ (file, "-"))
+  bool is_stdin = STREQ (file, "-");
+
+  if (is_stdin)
     {
-      fd = 0;
-      have_read_stdin = 1;
+      fd = STDIN_FILENO;
+      have_read_stdin = true;
+      if (O_BINARY && ! isatty (STDIN_FILENO))
+        xfreopen (NULL, "rb", stdin);
     }
   else
     {
-      fd = open (file, O_RDONLY);
+      fd = open (file, O_RDONLY | O_BINARY);
       if (fd == -1)
-	{
-	  error (0, errno, "%s", file);
-	  return -1;
-	}
+        {
+          error (0, errno, "%s", file);
+          return false;
+        }
     }
-  /* Need binary I/O, or else byte counts and checksums are incorrect.  */
-  SET_BINARY (fd);
 
   while (1)
     {
@@ -181,93 +184,90 @@ sysv_sum_file (const char *file, int print_name)
       size_t bytes_read = safe_read (fd, buf, sizeof buf);
 
       if (bytes_read == 0)
-	break;
+        break;
 
       if (bytes_read == SAFE_READ_ERROR)
-	{
-	  error (0, errno, "%s", file);
-	  if (!STREQ (file, "-"))
-	    close (fd);
-	  return -1;
-	}
+        {
+          error (0, errno, "%s", file);
+          if (!is_stdin)
+            close (fd);
+          return false;
+        }
 
       for (i = 0; i < bytes_read; i++)
-	s += buf[i];
+        s += buf[i];
       total_bytes += bytes_read;
     }
 
-  if (!STREQ (file, "-") && close (fd) == -1)
+  if (!is_stdin && close (fd) != 0)
     {
       error (0, errno, "%s", file);
-      return -1;
+      return false;
     }
 
   r = (s & 0xffff) + ((s & 0xffffffff) >> 16);
   checksum = (r & 0xffff) + (r >> 16);
 
   printf ("%d %s", checksum,
-	  human_readable (total_bytes, hbuf, human_ceiling, 1, 512));
+          human_readable (total_bytes, hbuf, human_ceiling, 1, 512));
   if (print_name)
     printf (" %s", file);
   putchar ('\n');
 
-  return 0;
+  return true;
 }
 
 int
 main (int argc, char **argv)
 {
-  int errors = 0;
+  bool ok;
   int optc;
   int files_given;
-  int (*sum_func) (const char *, int) = bsd_sum_file;
+  bool (*sum_func) (const char *, int) = bsd_sum_file;
 
   initialize_main (&argc, &argv);
-  program_name = argv[0];
+  set_program_name (argv[0]);
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
   atexit (close_stdout);
 
-  have_read_stdin = 0;
+  /* Line buffer stdout to ensure lines are written atomically and immediately
+     so that processes running in parallel do not intersperse their output.  */
+  setvbuf (stdout, NULL, _IOLBF, 0);
+
+  have_read_stdin = false;
 
   while ((optc = getopt_long (argc, argv, "rs", longopts, NULL)) != -1)
     {
       switch (optc)
-	{
-	case 0:
-	  break;
+        {
+        case 'r':		/* For SysV compatibility. */
+          sum_func = bsd_sum_file;
+          break;
 
-	case 'r':		/* For SysV compatibility. */
-	  sum_func = bsd_sum_file;
-	  break;
+        case 's':
+          sum_func = sysv_sum_file;
+          break;
 
-	case 's':
-	  sum_func = sysv_sum_file;
-	  break;
+        case_GETOPT_HELP_CHAR;
 
-	case_GETOPT_HELP_CHAR;
+        case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
 
-	case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
-
-	default:
-	  usage (EXIT_FAILURE);
-	}
+        default:
+          usage (EXIT_FAILURE);
+        }
     }
 
   files_given = argc - optind;
-  if (files_given == 0)
-    {
-      if ((*sum_func) ("-", files_given) < 0)
-	errors = 1;
-    }
+  if (files_given <= 0)
+    ok = sum_func ("-", files_given);
   else
-    for (; optind < argc; optind++)
-      if ((*sum_func) (argv[optind], files_given) < 0)
-	errors = 1;
+    for (ok = true; optind < argc; optind++)
+      ok &= sum_func (argv[optind], files_given);
 
   if (have_read_stdin && fclose (stdin) == EOF)
     error (EXIT_FAILURE, errno, "-");
-  exit (errors == 0 ? EXIT_SUCCESS : EXIT_FAILURE);
+  exit (ok ? EXIT_SUCCESS : EXIT_FAILURE);
 }
