@@ -1,5 +1,5 @@
 /* sync - update the super block
-   Copyright (C) 1994-2004, 2008-2009 Free Software Foundation, Inc.
+   Copyright (C) 1994-2004, 2008-2010 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ Force changed blocks to disk, update the super block.\n\
 "), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
-      emit_bug_reporting_address ();
+      emit_ancillary_info ();
     }
   exit (status);
 }
