@@ -1,5 +1,5 @@
 /* truncate -- truncate or extend the length of files.
-   Copyright (C) 2008 Free Software Foundation, Inc.
+   Copyright (C) 2008-2010 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,12 +18,7 @@
 
    This is backwards compatible with the FreeBSD utility, but is more
    flexible wrt the size specifications and the use of long options,
-   to better fit the "GNU" environment.
-
-   Note if !defined(HAVE_FTRUNCATE) then the --skip-ftruncate configure flag
-   was specified or we're in a mingw environment. In these cases gnulib
-   emulation will be used and GNULIB_FTRUNCATE is defined. Note if emulation
-   can't even be provided ftruncate() will return EIO.  */
+   to better fit the "GNU" environment.  */
 
 #include <config.h>             /* sets _FILE_OFFSET_BITS=64 etc. */
 #include <stdio.h>
@@ -117,17 +112,14 @@ Mandatory arguments to long options are mandatory for short options too.\n\
   -c, --no-create        do not create any files\n\
 "), stdout);
       fputs (_("\
-  -o, --io-blocks        Treat SIZE as number of IO blocks instead of bytes\n\
+  -o, --io-blocks        treat SIZE as number of IO blocks instead of bytes\n\
 "), stdout);
       fputs (_("\
   -r, --reference=FILE   use this FILE's size\n\
   -s, --size=SIZE        use this SIZE\n"), stdout);
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
-      fputs (_("\n\
-SIZE is a number which may be followed by one of the following suffixes:\n\
-KB 1000, K 1024, MB 1000*1000, M 1024*1024, and so on for G, T, P, E, Z, Y.\n\
-"), stdout);
+      emit_size_note ();
       fputs (_("\n\
 SIZE may also be prefixed by one of the following modifying characters:\n\
 `+' extend by, `-' reduce by, `<' at most, `>' at least,\n\
@@ -136,7 +128,7 @@ SIZE may also be prefixed by one of the following modifying characters:\n\
 \n\
 Note that the -r and -s options are mutually exclusive.\n\
 "), stdout);
-      emit_bug_reporting_address ();
+      emit_ancillary_info ();
     }
   exit (status);
 }
@@ -229,7 +221,7 @@ do_ftruncate (int fd, char const *fname, off_t ssize, rel_mode_t rel_mode)
       /* Complain only when ftruncate fails on a regular file, a
          directory, or a shared memory object, as POSIX 1003.1-2004
          specifies ftruncate's behavior only for these file types.
-         For example, do not complain when Linux 2.4 ftruncate
+         For example, do not complain when Linux kernel 2.4 ftruncate
          fails on /dev/fd0.  */
       int const ftruncate_errno = errno;
       if (fstat (fd, &sb) != 0)
@@ -287,7 +279,7 @@ main (int argc, char **argv)
 
         case 's':
           /* skip any whitespace */
-          while (isspace (*optarg))
+          while (isspace (to_uchar (*optarg)))
             optarg++;
           switch (*optarg)
             {
@@ -309,7 +301,7 @@ main (int argc, char **argv)
               break;
             }
           /* skip any whitespace */
-          while (isspace (*optarg))
+          while (isspace (to_uchar (*optarg)))
             optarg++;
           if (*optarg == '+' || *optarg == '-')
             {
@@ -416,9 +408,3 @@ main (int argc, char **argv)
 
   return errors ? EXIT_FAILURE : EXIT_SUCCESS;
 }
-
-/*
- * Local variables:
- *  indent-tabs-mode: nil
- * End:
- */
